import { AlertService, LoadingService, Modal, ModalService, Grid, TreeGrid } from '@zeedhi/common';
import { SocketEvent } from '@/socket/socket-event';
import {
  Config,
  Http,
  IDictionary,
  Metadata,
} from '@zeedhi/core';
import { io, Socket } from 'socket.io-client';

export class UpdatesController {
  public socket?: Socket;

  public title: any;

  public visibleCondition({ row }: IDictionary) {
    if (row.UPSISVERATUAL === '-' || row.UPSISVERATUAL <= '0.0.1') {
      return false;
    }
    return true;
  }

  /**
   * Condições para o icone de atualização
   * @param param0
   * @returns
   */
  public visibleConditionUpdate({ row }: IDictionary) {
    let up = false;
    if (
      row.UPSISVERATUAL === '-'
      || row.UPSISVERDISPONIVEL.length === 0
      || row.UPSISVERDISPONIVEL === undefined
      || row.UPSISVERDISPONIVEL === '-'
      || row.UPSISVERDISPONIVEL === 'Contate o administrador do sistema'
      || row.UPSISVERDISPONIVEL === 'Pacote sem informações da aplicação'
      || row.UPSISVERDISPONIVEL === 'Pacote sem informações de versão'
      || row.UPSISVERDISPONIVEL === false
      || row.UPSISVERDISPONIVEL.length >= 10
      ) {
      up = false;
    } else {
      up = true;
    }
    // console.log(up, row);
    if (row.VERSAOATUAL.UPVERSTATUS === 0) {
      up = false;
    }

    return up;
  }

  /**
   * Condições para o icone de Downgrade
   */
  public visibleConditionDowngrade({ row }: IDictionary) {
    if (row.UPSISVERATUAL === '-' || row.UPSISVERATUAL <= '0.0.1') {
      return false;
    }
    if (row.VERSAOATUAL.UPVERSTATUS === 0) {
      return false;
    }
    return true;
  }

  public visibleConditionStatus({ row }: IDictionary) {
    let cor;
    [row].forEach((element:any) => {
      if (element.SISTEMASSTATUS === '0' || element.SISTEMASSTATUS === 0
          || (element.UPSISVERDISPONIVEL && element.UPSISVERDISPONIVEL.length >= 10 && !element.UPSISVERDISPONIVEL.includes('Atualização pendente para a versão'))) {
        cor = 'red';
      } else {
        if (element.UPSISVERDISPONIVEL !== undefined) {
          cor = 'yellow';
        } if (element.UPSISVERDISPONIVEL === '-') {
          cor = 'green';
        }
        if (element.VERSAOATUAL.UPVERSTATUS === 0 && element.UPSISVERDISPONIVEL !== '-') {
          cor = 'gray';
        }
        if (element.UPSISVERDISPONIVEL !== undefined && element.UPSISVERDISPONIVEL.length >= 6) {
          cor = 'gray';
        }
      }
    });

    return cor;
  }

  // public startModal() {
  //   Loader.getInstance('HistoricoController').createFullscreenModal();
  // }

  private fullscreenModal!: Modal;

  public async createFullscreenModal() {
    this.fullscreenModal = await ModalService.createFromJson('historico', true);
  }

  public destroyFullscreenModal() {
    this.fullscreenModal?.destroy();
  }

  public openFullscreenModal() {
    this.fullscreenModal.show();
  }

  public closeFullscreenModal() {
    this.fullscreenModal.hide();
  }

  private startUpdater(component: any) {
    component.datasource.loading = true;

    // inicializa o socket
    this.socket = io(Config.endPoint);
    this.socket.emit(SocketEvent.CONNECTION)
      .emit('login', {
        email: 'francismar.araujo@teknisa.com',
        password: 'nineninenine',
        slave: false,
      }, {});

    this.socket
      .on('confirmLogin', async (branc: IDictionary) => {
        if (branc) {
          console.warn();
        }
      });

    // setInterval(() => {

    // }, 5000);
    this.socket
        .emit('updater', {
          slave: false,
          args: {
            interval: 10000,
          },
        });

    this.socket
      .on('updater', async (branc: IDictionary) => {
        // console.log('branc: ', branc);
        await component.datasource.updateData(branc.message.data);
        await component.datasource.get();
        component.datasource.loading = false;
      });

    setTimeout(() => {
      if (this.socket?.id === undefined) {
        component.datasource.loading = false;
        this.showAlert('Não foi possível conectar ao servidor', 'error');
      }
    }, 1000);
  }

  /**
   * Método executado quando o grid é criado
   */
  public gridMounted(force:boolean = false) {
    const grid = Metadata.getInstance<TreeGrid>('grid');
    const date = new Date().toLocaleDateString();
    const savedDate = localStorage.getItem('LAST_DATE_UPDATER');
    if (savedDate !== date || (typeof force === 'boolean' && force)) {
      LoadingService.show('Atualizando lista de sistemas...');
      const conexao = `${Config.metadataEndPoint}orgProducts`;
      Http.get(conexao).then((result) => {
        if (result.data.isError) {
          this.showAlert(`Erro ao atualizar a lista de sistemas: ${result.data.response}`, 'error');
        } else {
          this.showAlert('Lista de sistemas atualizada com sucesso!', 'success');
        }
      }).catch((error) => {
        this.showAlert(error, 'error');
      }).finally(() => {
        LoadingService.hide();
        localStorage.setItem('LAST_DATE_UPDATER', date);
        this.startUpdater(grid);
      });
    } else {
      this.startUpdater(grid);
    }
  }

  public updateSystemList() {
    this.gridMounted(true);
  }

  private showAlert(message: string, colorStr: string) {
    AlertService.show({
      name: 'info-alert',
      text: message,
      color: colorStr,
      right: true,
      top: true,
      showDismiss: true,
      timeout: 10000,
    });
  }

  public selectClick({ component }: any) {
    // const itemsText = component.selectedRows.map((item: any) => item.id).join(', ');
    // component.getChildInstance('selected-rows').text = `selectedRows = [${itemsText}]`;
    console.log(component);
  }

  /**
   * @author daniel.souza
   * @param element elemento clicado pelo usuário
   */
  public updaters(element:any) {
    LoadingService.show();
    this.socket = io(Config.endPoint);
    this.socket.emit(SocketEvent.CONNECTION);
    console.log(element);

    element.row.UPSISVERDISPONIVEL = '-';

    this.socket
      .emit('updaters', {
        slave: false,
        dados: {
          id: element.row.id,
          IDUPDATER: element.row.IDUPDATER,
          CDFILIAL: element.row.CDFILIAL,
          NOMEFILIAL: element.row.NOMEFILIAL,
          IDUPDATERSISTEMAS: element.row.IDUPDATERSISTEMAS,
          DEVOPSID: element.row.DEVOPSID,
          APLICACAO: element.row.APLICACAO,
          SERVICO: element.row.SERVICO,
          UPSISVERATUAL: element.row.UPSISVERATUAL,
          SISTEMASSTATUS: element.row.SISTEMASSTATUS,
        },
        operador: localStorage.getItem('CDOPERADOR'),
        tipo: element.column.name,
      });

    this.socket
      .on('updaters', async (branc: IDictionary) => {
        LoadingService.hide();
        if (branc.error === true) {
          this.showAlert(`Erro ao registrar o pedido de atualização do produto: ${branc.message.data}`, 'error');
        } else {
          this.showAlert('Pedido de atualização cadastrado com sucesso!', 'success');
          element.row.UPSISVERDISPONIVEL = '-';
          const grid = Metadata.getInstance<TreeGrid>('grid');
          grid.datasource.get();
          /** Reload */
          document.location.reload();
        }
        console.warn('Branch:', branc);
      });
  }

  get disabled(): boolean {
    const grid = Metadata.getInstance<Grid>('grid');
    return grid.selectedRows.length <= 0;
  }

  /**
   * Envia solicitação para atualizar os sistemas selecionados
   * @author daniel.souza
   * @param param0
   */
  public updateSelected({ component }: any) {
    const grid: any = component.parent;
    const itens:any = [];

    /** Valida se tem atualização disponível */
    grid.selectedRows.forEach((item: any) => {
      if (
        item.UPSISVERATUAL === '-'
        || item.UPSISVERDISPONIVEL.length === 0
        || item.UPSISVERDISPONIVEL === undefined
        || item.UPSISVERDISPONIVEL === '-'
        || item.UPSISVERDISPONIVEL === 'Contate o administrador do sistema'
        || item.UPSISVERDISPONIVEL === 'Pacote sem informações da aplicação'
        || item.UPSISVERDISPONIVEL === 'Pacote sem informações de versã'
      ) {
        // console.log('item: ', item.UPSISVERDISPONIVEL);
      } else {
        itens.push(item);
      }
    });

    this.socket = io(Config.endPoint);
    this.socket.off(SocketEvent.CONNECTION)
      .emit('updaters', {
        slave: false,
        dados: itens,
        operador: localStorage.getItem('CDOPERADOR'),
        tipo: 'up',
      });

    this.socket.off(SocketEvent.CONNECTION)
      .on('updaters', async (branc: IDictionary) => {
        console.warn('Branch:', branc);
      });

    /** Modifica o valor do Grid para garantir que não terá uma nova solicitação */
    itens.forEach((row: any) => {
      row.UPSISVERDISPONIVEL = '-';
    });
    grid.datasource.updateData(grid.data);
  }

  /**
   * Método executado logo antes do grid ser destruído
   */
  public gridBeforeDestroy() {

  }
}
